#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import json
import os
from pathlib import Path

import requests

config_path = Path("~/.notify-slack-cfg").expanduser()
if not config_path.exists():
    error_msg = """
    ERROR: A Webhook URL is required for running notify-slack.
    Create yours here: https://my.slack.com/services/new/incoming-webhook/
    Once you have your KEY, please create a file at ${HOME}/.notify-slack-cfg containng only the KEY. Eg: T02TKKSAX/B246MJ6HX/WXt2BWPfNhSKxdoFNFblczW9
    """
    raise ValueError(error_msg)

SLACK_NOTIFY_ICON = ":ubuntu:"
SLACK_BOT_USERNAME = "Bash Notifier"
SLACK_WEBHOOK_SERVICE = config_path.read_text().strip()
SLACK_URL = f"https://hooks.slack.com/services/{SLACK_WEBHOOK_SERVICE}"
LOG_PATH = Path("DLA_train.log")
DLA_LENGTH = 18

ICONS = {
    "INFO": ":information_source: ",
    "WARN": ":warning: ",
    "ERROR": ":github_changes_requested: ",
}


def get_icon(message):
    for log_prefix, icon in ICONS.items():
        if message.startswith(log_prefix):
            return icon
    return ""


def read_log(log_file, number_of_lines):
    if log_file.exists() and number_of_lines > 0:
        # read `number_of_lines` last lines of the log file (to get the CER and WER if possible)
        lines = log_file.read_text().split("\n")[-number_of_lines:]
        if lines:
            return f"{log_file}:\n```" + "\n".join(lines) + "```"
    else:
        print(f"{log_file} doesn't exist")
    return ""


def run(message, log_file, number_of_lines):
    icon = get_icon(message)
    dla_log = read_log(log_file, number_of_lines)
    payload = {
        "text": f"{icon}{message}\n{dla_log} from {os.uname().nodename}:{Path().cwd()}",
        "icon_emoji": SLACK_NOTIFY_ICON,
        "username": SLACK_BOT_USERNAME,
    }
    res = requests.post(
        SLACK_URL,
        data=json.dumps(payload),
        # headers={'Content-Type':'application/x-www-form-urlencoded'}
    )
    assert res.status_code == 200


def main():
    parser = argparse.ArgumentParser("Send message to slack")
    parser.add_argument("message", help="Message to be sent to slack", type=str)
    parser.add_argument(
        "--log_file",
        help="Log file from where last N files will be included with the message. "
        "Use None as file name if don't want to include a log file.",
        type=Path,
        default=LOG_PATH,
    )
    parser.add_argument(
        "-N",
        "--number_of_lines",
        help="Number of lines to be included from the end of the log file."
        "Use 0 to not include a log file.",
        type=int,
        default=DLA_LENGTH,
    )
    args = parser.parse_args()
    run(**vars(args))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import hashlib
import os

import requests
import yaml

assert os.environ.get("CI"), "Can only run on Gitlab CI"
TOKEN = os.environ.get("CI_JOB_TOKEN")
assert TOKEN, "Missing CI token"
PROJECT_ID = os.environ.get("CI_PROJECT_ID")
assert PROJECT_ID, "Missing CI project ID"


def md5sum(path):
    """
    Calc the MD5 hash of a binary file
    """
    with open(path, mode="rb") as f:
        d = hashlib.md5()
        while True:
            buf = f.read(4096)
            if not buf:
                break
            d.update(buf)
        return d.hexdigest()


def remote_parameters(name, version):
    """
    Fetch the parameters of a pre-existing model
    stored on the package registry
    """
    headers = {
        "JOB-TOKEN": TOKEN,
    }
    url = f"https://gitlab.teklia.com/api/v4/projects/{PROJECT_ID}/packages/generic/{name}/{version}/parameters.yml"
    r = requests.get(url, headers=headers)
    if not r.ok:
        # Normal case where the model is not available
        if r.status_code == 404:
            return
        else:
            r.raise_for_status()

    # Parse the parameters as YAML
    return yaml.safe_load(r.content)


def upload(path, name, version):
    """
    Upload any file on the Gitlab generic package registry
    """
    headers = {
        "JOB-TOKEN": TOKEN,
    }
    filename = os.path.basename(path)
    url = f"https://gitlab.teklia.com/api/v4/projects/{PROJECT_ID}/packages/generic/{name}/{version}/{filename}"
    r = requests.put(url, headers=headers, data=open(path, "rb"))
    r.raise_for_status()
    print(f"Uploaded {path}")


def publish_model(model_name):
    """
    Publish a single model on the Gitlab generic registry
    - will publish model.pth & parameters.yml
    - only if they are not already published for that version & model hash
    """

    # Check required files are present
    model_path = os.path.join("models", model_name, "model.pth")
    parameters_path = os.path.join("models", model_name, "parameters.yml")
    assert os.path.exists(model_path), f"Missing model {model_path}"
    assert os.path.exists(parameters_path), f"Missing parameters {parameters_path}"

    # Parse parameters
    parameters = yaml.safe_load(open(parameters_path))
    version = parameters.get("version")
    assert version, f"Missing version in {parameters_path}"
    print(f"Publishing {model_name} @ {version}")

    # Hash model and update the parameters file
    parameters["md5sum"] = md5sum(model_path)

    # Retrieve remote parameters to check if they differ
    if parameters == remote_parameters(model_name, version):
        print("Model is already available, skipping.")
        return

    # Add version to the parameters
    with open(parameters_path, "w") as f:
        yaml.safe_dump(parameters, f)

    # Upload the model on generic package registry
    upload(model_path, model_name, version)
    upload(parameters_path, model_name, version)


if __name__ == "__main__":
    for p in os.listdir("models"):
        publish_model(p)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    The retrieve experiments configs module
    ======================

    Use it to get the configurations for running the experiments.
"""

import argparse
import csv
import json
import logging
import os
from pathlib import Path

logger = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


STAGE_TRAIN = "train"
STAGE_VAL = "val"
STAGE_TEST = "test"

CONVERT = {"images": "image", "labels": "mask", "labels_json": "json"}


def generate_configurations(csv_path):
    """
    Read configuration references from a CSV and generate a configuration
    for each line, using related local files
    """
    assert csv_path.exists(), f"Missing CSV {csv_path}"

    workdir = csv_path.parent.resolve()

    reader = csv.DictReader(csv_path.open(), delimiter=",")
    for row in reader:
        config = {}
        # Get experiment name.
        assert row["experiment_name"] != "", "Missing experiment name"
        config["experiment_name"] = row["experiment_name"]

        # Get steps as a list of names.
        config["steps"] = row["steps"].split(";")

        # Get train/val/test folders.
        config["data_paths"] = {}
        for dataset in [STAGE_TRAIN, STAGE_VAL, STAGE_TEST]:
            config["data_paths"][dataset] = {}

            # Handle conversion table, no labels when running tests
            conversion_table = dict(CONVERT)
            if dataset == STAGE_TEST:
                del conversion_table["labels"]

            for folder, key in CONVERT.items():
                if row[dataset] != "":
                    config["data_paths"][dataset][key] = [
                        (workdir / element / dataset / folder).as_posix()
                        for element in row[dataset].split(";")
                    ]
                else:
                    config["data_paths"][dataset][key] = []

        # Get restore model.
        if row["restore_model"] != "":
            config["training"] = {"restore_model": row["restore_model"]}
            if row["same_classes"] != "":
                config["training"]["same_classes"] = row["same_classes"]
            if row["loss"] != "":
                config["training"]["loss"] = row["loss"]

        yield config


def run(csv_path, output):
    """
    Retrieve the configurations for the experiments from a config csv file.
    Save each configuration into TMP_DIR/experiment_name file.
    """
    os.makedirs(output, exist_ok=True)

    for index, config in enumerate(generate_configurations(csv_path), 1):
        # Save each configuration in a dedicated file file.
        json_file = str(index) + "_" + config["experiment_name"] + ".json"
        with open(os.path.join(output, json_file), "w") as file:
            json.dump(
                {key: value for key, value in config.items() if value},
                file,
                indent=4,
            )

    logger.info(
        f"Retrieved {index} experiment configurations from {csv_path}"
        if index > 1
        else f"Retrieved {index} experiment configuration from {csv_path}"
    )


def main():
    """
    Main function.
    """
    parser = argparse.ArgumentParser(
        description="Script to retrieve the experiments configs.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "config", type=Path, help="Path to the configurations file (CSV format)"
    )
    parser.add_argument(
        "--output",
        type=Path,
        default=Path("./tmp"),
        help="Directory where the generated configurations will be stored",
    )

    args = parser.parse_args()
    run(args.config, args.output)


if __name__ == "__main__":
    main()
